/*	GUI for Petri-net Tool
   	Author Daniel Justice
	22/01/02
*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class Gui {
	
	private JPanel myPanel;
	private JFrame myGui;
	private	ActionListener listener;
	private Container mainContentPane;
	private JMenu fileMenu, editMenu, insertMenu, moduleMenu, preferenceMenu, helpMenu;
	private JMenu configureSubMenu, gridSubMenu;
	private JFrame mainFrame;

	// Constructor for class GUI
	public Gui() {

		myGui = new JFrame("Petri-net Tool");
		listener = new MenuItemActionListener(myPanel);
		myGui.setJMenuBar(createMenuBar(listener));

		mainContentPane = myGui.getContentPane();
		mainContentPane.setLayout( new BorderLayout() );
    		mainContentPane.add( createToolBar(listener), BorderLayout.NORTH );
		mainContentPane.add(createDesktop(), BorderLayout.CENTER);
			
		// set default window size
		int inset = 100;
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		myGui.setBounds(inset, inset, screenSize.width - inset*2, screenSize.height - inset*2);

    		myGui.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
 	  	myGui.show();
	}

	// Function to produce the Menu Bar
	private JMenuBar createMenuBar(ActionListener listener) {
	
		JMenuBar menuBar = new JMenuBar();
	
		fileMenu = newMenu("File", 'F');
		editMenu = newMenu("Edit", 'E');
		insertMenu = newMenu("Insert", 'I');
		moduleMenu = newMenu("Modules", 'M');
		preferenceMenu = newMenu("Preferences", 'P');
		helpMenu = newMenu("Help", 'H');

		fileMenu.add(newMenuItem("New", listener, "new", 'N', KeyEvent.VK_N));
		fileMenu.addSeparator();
		fileMenu.add(newMenuItem("Open", listener, "open", 'O', KeyEvent.VK_O));
		fileMenu.add(newMenuItem("Save", listener, "save", 'S', KeyEvent.VK_S));		
		fileMenu.add(newMenuItem("Save As..", listener, "saveAs", 'V', 0));
		fileMenu.addSeparator();
		fileMenu.add(newMenuItem("Print", listener, "print", 'P', 0));
		fileMenu.addSeparator();
		fileMenu.add(newMenuItem("Exit", listener, "exit", 'x', 0));
	
		editMenu.add(newMenuItem("Undo", listener, "undo", 'U', KeyEvent.VK_U));
		editMenu.add(newMenuItem("Redo", listener, "redo", 0, 0));
		editMenu.addSeparator();
		editMenu.add(newMenuItem("Cut", listener, "cut", 'C', KeyEvent.VK_X));
		editMenu.add(newMenuItem("Copy", listener, "copy", 'O', KeyEvent.VK_C));
		editMenu.add(newMenuItem("Paste", listener, "paste", 'P', KeyEvent.VK_V));
		editMenu.addSeparator();
		editMenu.add(newMenuItem("Select All", listener, "selectAll", 'S', 0));

		insertMenu.add(newMenuItem("Place", new ImageIcon("Images/place.gif"), listener, "place", 'P'));
		insertMenu.add(newMenuItem("Immediate Transition", new ImageIcon("Images/immediate.gif"),
					   listener, "immediate", 'I'));
		insertMenu.add(newMenuItem("Timed Transition", new ImageIcon("Images/timed.gif"), listener, "timed", 'T'));
		insertMenu.add(newMenuItem("Arc", new ImageIcon("Images/arc.gif"), listener, "arc", 'A'));
		insertMenu.add(newMenuItem("Add Token", new ImageIcon("Images/addtok.gif"), listener, "addtok", 'O'));
		insertMenu.add(newMenuItem("Remove Token", new ImageIcon("Images/remtok.gif"), listener, "remtok", 'R'));
		insertMenu.add(newMenuItem("Subnet", new ImageIcon("Images/subnet.gif"), listener, "subnet", 'S'));

		moduleMenu.add(newMenuItem("Load Module", listener, "loadMod", 'L', 0));

		configureSubMenu = newMenu("Edit Colours", 'E');
		configureSubMenu.add(newMenuItem("Background Colour", listener, "background", 'B', 0));
		configureSubMenu.add(newMenuItem("Foreground Colour", listener, "foreground", 'F', 0));
		gridSubMenu = newMenu("Edit Grid", 'G');
		gridSubMenu.add( new JCheckBoxMenuItem("Display Grid"));
		gridSubMenu.add( new JCheckBoxMenuItem("Snap To Grid"));
		 	
		preferenceMenu.add(newMenuItem("Default Configuration", listener, "default", 'D', 0));
		preferenceMenu.addSeparator();
		preferenceMenu.add(gridSubMenu);
		preferenceMenu.add(configureSubMenu);

		helpMenu.add(newMenuItem("About..", listener, "about", 'A', 0));
			
		menuBar.add(fileMenu);
		menuBar.add(editMenu);
		menuBar.add(insertMenu);
		menuBar.add(moduleMenu);
		menuBar.add(preferenceMenu);
		menuBar.add(helpMenu);

		return menuBar;
	}

	// Function to produce the toolbar
	private JToolBar createToolBar(ActionListener listener) {

		JToolBar toolBar = new JToolBar();
	
		toolBar.add( newButton( "Images/filenew.gif", listener, "New", "new" ));
		toolBar.add( newButton( "Images/fopen.gif", listener, "Open", "open" ));
		toolBar.add( newButton( "Images/fsave.gif", listener, "Save", "save" ));
		toolBar.add( newButton( "Images/print.gif", listener, "Print", "print" ));
		toolBar.addSeparator();

		toolBar.add( newButton( "Images/cut.gif", listener, "Cut", "cut" ));
		toolBar.add( newButton( "Images/copy.gif", listener, "Copy", "copy" ));
		toolBar.add( newButton( "Images/paste.gif", listener, "Paste", "paste" ));
		toolBar.addSeparator();

		toolBar.add( newButton( "Images/place.gif", listener, "Place", "place" ));
		toolBar.add( newButton( "Images/immediate.gif", listener, "Immediate Transition", "immediate" ));
		toolBar.add( newButton( "Images/timed.gif", listener, "Timed Trasition", "timed" ));
		toolBar.add( newButton( "Images/arc.gif", listener, "Arc", "arc" ));
		toolBar.add( newButton( "Images/addtok.gif", listener, "Add Token", "addtok" ));
		toolBar.add( newButton( "Images/remtok.gif", listener, "Remove Token", "remtok" ));
		toolBar.add( newButton( "Images/subnet.gif", listener, "Subnet", "subnet" ));
		toolBar.add( newButton( "Images/save.gif", listener, "Rotate", "rotate" ));

		return toolBar;
	
	}

	// Function to create DeskTop
	private JDesktopPane createDesktop() {

		JInternalFrame internalFrame = new JInternalFrame("Net-diagram");
		internalFrame.setSize(400,400);
		internalFrame.setResizable(true);
		internalFrame.setMaximizable(true);

		JDesktopPane desktop = new JDesktopPane();
		desktop.setBackground(Color.white);
		desktop.add(internalFrame);
		desktop.setBorder( BorderFactory.createLoweredBevelBorder());
		internalFrame.setBackground(Color.lightGray);
		internalFrame.show();

		return desktop;

	}

	// Event listener class
	private static class MenuItemActionListener implements ActionListener {

		Component parent;
		public MenuItemActionListener( Component parent ) {
			this.parent = parent; }

		public void actionPerformed( ActionEvent e ) {
			// CODE TO HANDLE EVENTS HERE
		}
	}
				
	// Method for creating normal JMenuItems
	private static JMenuItem newMenuItem( String label, ActionListener listener, String command,
						     int mnemonic, int acceleratorKey) {

		JMenuItem item = new JMenuItem(label);			
		item.addActionListener(listener);
		item.setActionCommand(command);
		if (mnemonic != 0)
			item.setMnemonic((char) mnemonic);
		if (acceleratorKey != 0)
			item.setAccelerator(KeyStroke.getKeyStroke(acceleratorKey, java.awt.Event.CTRL_MASK));

		return item;
	}

	// Method for creating JMenuItems for Places, Transitions etc.
	private static JMenuItem newMenuItem( String label, ImageIcon image, ActionListener listener, String command,
					      int mnemonic) {
		
		JMenuItem item = new JMenuItem(label, image);			
		item.addActionListener(listener);
		item.setActionCommand(command);
		if (mnemonic != 0)
			item.setMnemonic((char) mnemonic);

		return item;
	}


	// Method for creating JMenus
	private static JMenu newMenu(String label, int mnemonic) {

		JMenu item = new JMenu(label);
		if (mnemonic != 0)
			item.setMnemonic((char) mnemonic);
		
		return item;
	}

	// Method for creating buttons.
  	private static JButton newButton( String icon, ActionListener listener, String tooltip,
                                      String command ) {

	  	JButton item = new JButton( new ImageIcon( icon ));
    		item.addActionListener( listener );
    		item.setToolTipText( tooltip );
    		item.setActionCommand( command );

		return item;
  	}


	
}

class Demo
{ public static void main( String param[] ) throws Exception
  { new Gui();
  }
}


